using Jypeli;

namespace TippuKivet
{
    public class TippuKivet : PhysicsGame
    {
        // Pelaajan liikkumisnopeudet oikealle ja vasemmalle
        private readonly Vector nopeusOikea = new Vector(800, 0);
        private readonly Vector nopeusVasen = new Vector(-800, 0);

        // Pelin keskeiset muuttujat
        private PhysicsObject pelaaja;
        private IntMeter pistelaskuri; // Pisteiden laskuri
        private Timer ajastinTimantit; // Ajastin timanttien luomiselle
        private Timer ajastinKivet; // Ajastin kivien luomiselle
        private int ohimenneetTimantit; // Laskuri, montako timanttia pelaaja on missannut

        public override void Begin()
        {
            // Asetetaan pelin ikkunan koko
            SetWindowSize(800, 600, false);

            // Aloitetaan timanttien ja kivien generointi
            ajastinTimantit = Timer.CreateAndStart(1.5, LuoTimantti);
            ajastinKivet = Timer.CreateAndStart(3.0, LuoKivi);

            // Alustetaan pistelaskuri
            pistelaskuri = new IntMeter(0);
            LuoPisteNaytto();

            // Asetetaan painovoima
            Gravity = new Vector(0, -700);

            // Luodaan pelaaja ja pelin kenttä
            pelaaja = LuoPelaaja(0);
            LuoKenttä();

            // Asetetaan ohjaimet
            AsetaOhjaimet();
        }

        private void LuoKenttä()
        {
            // Luodaan näkymätön alalaatta, johon kivet ja timantit voivat osua
            PhysicsObject alalaatta = Level.CreateBottomBorder();
            alalaatta.IsVisible = false;
            AddCollisionHandler(alalaatta, "Kivi", KiviAlalaattaCollide);
            AddCollisionHandler(alalaatta, "Timantti", TimanttiAlalaattaCollide);

            // Asetetaan taustakuva ja taustan väri
            Level.Background.Image = LoadImage("taustakuva");
            Level.Background.Color = Color.Black;

            // Zoomataan kamera kenttään sopivaksi
            Camera.ZoomToLevel();
        }

        // Luodaan pelaaja-objekti
        private PhysicsObject LuoPelaaja(double x)
        {
            PhysicsObject pelaaja = LuoObjekti(150, 200, x, -300, Color.Red, "Pelaaja");
            pelaaja.IgnoresCollisionResponse = true; // Törmäykset Pelaajaan ei vaikuta.
            pelaaja.IgnoresGravity = true; // Painovoima ei vaikuta Pelaajaan
            pelaaja.Image = LoadImage("pelaaja");

            // Lisätään törmäyskäsittelijät pelaajalle
            AddCollisionHandler(pelaaja, "Kivi", PelaajaKiviCollide);
            AddCollisionHandler(pelaaja, "Timantti", PelaajaTimanttiCollide);
            Add(pelaaja);
            return pelaaja;
        }

        // Luodaan pistelaskuri-näyttö
        private void LuoPisteNaytto()
        {
  
            Label pisteNaytto = new Label
            {
                Position = new Vector(340, 250),
                TextColor = Color.Red,
                Color = Color.White,
                Size = new Vector(5000, 5000),
                Title = "Pisteet: ",
            };
            pisteNaytto.BindTo(pistelaskuri);
            Add(pisteNaytto);
        }

        // Luodaan satunnaiseen paikkaan ilmestyvä timantti
        private void LuoTimantti()
        {

            double x = RandomGen.NextDouble(Level.Left + 50, Level.Right - 50);
            PhysicsObject timantti = LuoObjekti(60, 60, x, Level.Top + 50, Color.Yellow, "Timantti");
            timantti.AngularVelocity = RandomGen.NextDouble(-5.0, 5.0);
            timantti.Image = LoadImage("timantti.png");
            Add(timantti);
        }

        private void LuoKivi()
        {
            // Luodaan satunnaiseen paikkaan ilmestyvä kivi
            double x = RandomGen.NextDouble(Level.Left + 20, Level.Right - 20);
            double randomSize = RandomGen.NextDouble(30.0, 120.0);
            PhysicsObject kivi = LuoObjekti(randomSize, randomSize, x, Level.Top + 50, Color.White, "Kivi");
            kivi.Velocity = new Vector(0, -400);
            kivi.Image = LoadImage("kivi.png");
            Add(kivi);
        }
        
        // Aliohjelma objektien luomiseen
        private PhysicsObject LuoObjekti(double width, double height, double x, double y, Color color, string tag)
        {
            return new PhysicsObject(width, height, Shape.Rectangle)
            {
                X = x,
                Y = y,
                Color = color,
                Tag = tag
            };
        }

        private void PelaajaTimanttiCollide(PhysicsObject pelaaja, PhysicsObject timantti)
        {
            pistelaskuri.Value += 1;
            timantti.Destroy();
            KutistaPelaaja();

            if (pistelaskuri.Value >= 10)
            {
                VoitaPeli();
            }
        }
        
        // Pelaajan ja kiven törmäyskäsittelijä
        private void PelaajaKiviCollide(PhysicsObject pelaaja, PhysicsObject kivi)
        {
            kivi.Destroy();
            LopetaPeli("Kivi osui sinua päähän, Peli ohi!");
        }

        // Kiven ja alalaatan törmäyskäsittelijä
        private void KiviAlalaattaCollide(PhysicsObject alalaatta, PhysicsObject kivi)
        {
            kivi.Destroy();
        }
        
        // Timantin ja alalaatan törmäyskäsittelijä
        private void TimanttiAlalaattaCollide(PhysicsObject alalaatta, PhysicsObject timantti)
        {
            timantti.Destroy();
            ohimenneetTimantit++;

            if (ohimenneetTimantit >= 2)
            {
                LopetaPeli("Et ollut tarpeeksi tehokas!");
            }
        }
        
        // Pelaajan koko pienenee timantin keräämisen jälkeen
        private void KutistaPelaaja()
        {

            const double shrinkFactor = 0.9;
            const double minSize = 10.0;

            if (pelaaja.Width > minSize && pelaaja.Height > minSize)
            {
                pelaaja.Width *= shrinkFactor;
                pelaaja.Height *= shrinkFactor;
            }
        }

        // Peli päättyy voittoon, jos pelaaja kerää tarpeeksi pisteitä
        private void VoitaPeli()
        {
            MessageDisplay.Add("Olet TippuKiviLuolan suvereeni Päällikkö!");
            ajastinTimantit.Stop();
            ajastinKivet.Stop();
            Timer.SingleShot(6, Exit);
            Pause();
        }
        
        // Peli päättyy häviöön, jos pelaaja osuu kiveen tai missaa liikaa timantteja
        private void LopetaPeli(string viesti)
        {
            MessageDisplay.Add(viesti);
            ajastinTimantit.Stop();
            ajastinKivet.Stop();
            Timer.SingleShot(7, Exit);
            Pause();
        }
        
        // Määritellään pelin näppäimet, pelaajan liike, pause ja ESC.
        private void AsetaOhjaimet()
        {

            Keyboard.Listen(Key.Left, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa vasemmalle", pelaaja, nopeusVasen);
            Keyboard.Listen(Key.Left, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Right, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa oikealle", pelaaja, nopeusOikea);
            Keyboard.Listen(Key.Right, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
            Keyboard.Listen(Key.P, ButtonState.Pressed, Pause, "Pysäyttää pelin");
        }

        private void AsetaNopeus(PhysicsObject pelaaja, Vector nopeus)
        {
            // Estetään pelaajaa menemästä kentän ulkopuolelle
            if ((pelaaja.X + pelaaja.Width / 2 >= Level.Right && nopeus.X > 0) ||
                (pelaaja.X - pelaaja.Width / 2 <= Level.Left && nopeus.X < 0))
            {
                pelaaja.Velocity = Vector.Zero;
            }
            else
            {
                pelaaja.Velocity = nopeus;
            }
        }
    }
}
